/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "common.h"

const char* GetErrString(ERR eErr,OUTPUT_PARAMETER char pcErrString[ERR_STRING_BUFSIZE])
{
    switch(eErr) {
    case ERR_NONE:              (void)strncpy(pcErrString,"ERR_NONE",ERR_STRING_BUFSIZE); break;
    case ERR_SEQ:               (void)strncpy(pcErrString,"ERR_SEQ",ERR_STRING_BUFSIZE); break;
    case ERR_COMMAND:           (void)strncpy(pcErrString,"ERR_COMMAND",ERR_STRING_BUFSIZE); break;
    case ERR_BOARD_FLASH:       (void)strncpy(pcErrString,"ERR_BOARD_FLASH",ERR_STRING_BUFSIZE); break;
    case ERR_PEB_NUMBER:        (void)strncpy(pcErrString,"ERR_PEB_NUMBER",ERR_STRING_BUFSIZE); break;
    case ERR_SUBSECTOR_NUMBER:  (void)strncpy(pcErrString,"ERR_SUBSECTOR_NUMBER",ERR_STRING_BUFSIZE); break;
    case ERR_ERASE:             (void)strncpy(pcErrString,"ERR_ERASE",ERR_STRING_BUFSIZE); break;
    case ERR_READ:              (void)strncpy(pcErrString,"ERR_READ",ERR_STRING_BUFSIZE); break;
    case ERR_WRITE:             (void)strncpy(pcErrString,"ERR_WRITE",ERR_STRING_BUFSIZE); break;
    case ERR_FLASH_UNVERIFIED:  (void)strncpy(pcErrString,"ERR_FLASH_UNVERIFIED",ERR_STRING_BUFSIZE); break;
    case ERR_PEB_SIZE:          (void)strncpy(pcErrString,"ERR_PEB_SIZE",ERR_STRING_BUFSIZE); break;
    case ERR_SUBSECTOR_SIZE:    (void)strncpy(pcErrString,"ERR_SUBSECTOR_SIZE",ERR_STRING_BUFSIZE); break;
    case ERR_IMAGE_SIZE:        (void)strncpy(pcErrString,"ERR_IMAGE_SIZE",ERR_STRING_BUFSIZE); break;
    case ERR_CHECKSUM:          (void)strncpy(pcErrString,"ERR_CHECKSUM",ERR_STRING_BUFSIZE); break;
    case ERR_MULTIBOOT_BYTES:   (void)strncpy(pcErrString,"ERR_MULTIBOOT_BYTES",ERR_STRING_BUFSIZE); break;
    case ERR_IMAGE_ALIGNMENT:   (void)strncpy(pcErrString,"ERR_IMAGE_ALIGNMENT",ERR_STRING_BUFSIZE); break;
    case ERR_BSP:               (void)strncpy(pcErrString,"ERR_BSP",ERR_STRING_BUFSIZE); break;
    case ERR_TIMEOUT:           (void)strncpy(pcErrString,"ERR_TIMEOUT",ERR_STRING_BUFSIZE); break;
    case ERR_INTERNAL:          (void)strncpy(pcErrString,"ERR_INTERNAL",ERR_STRING_BUFSIZE); break;
    default:                    (void)snprintf(pcErrString,ERR_STRING_BUFSIZE,"ERR_\?\?\?(0x%08x)",(unsigned int)eErr); break;
    }
    pcErrString[ERR_STRING_BUFSIZE - 1] = 0;
    return (const char*)pcErrString;
}

const char* GetCommandString(COMMAND eCommand,OUTPUT_PARAMETER char pcCommandString[COMMAND_STRING_BUFSIZE])
{
    switch(eCommand) {
    case COMMAND_INVALID:               (void)strncpy(pcCommandString,"COMMAND_INVALID",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_VERIFY_FLASH:          (void)strncpy(pcCommandString,"COMMAND_VERIFY_FLASH",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_VERIFY_IMAGE:          (void)strncpy(pcCommandString,"COMMAND_VERIFY_IMAGE",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_GET_MULTIBOOT_BYTES:   (void)strncpy(pcCommandString,"COMMAND_GET_MULTIBOOT_BYTES",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_ERASE_PEBS:            (void)strncpy(pcCommandString,"COMMAND_ERASE_PEBS",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_READ_PEB:              (void)strncpy(pcCommandString,"COMMAND_READ_PEB",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_WRITE_PEB:             (void)strncpy(pcCommandString,"COMMAND_WRITE_PEB",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_ERASE_SUBSECTOR:       (void)strncpy(pcCommandString,"COMMAND_ERASE_SUBSECTOR",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_READ_SUBSECTOR:        (void)strncpy(pcCommandString,"COMMAND_READ_SUBSECTOR",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_WRITE_SUBSECTOR:       (void)strncpy(pcCommandString,"COMMAND_WRITE_SUBSECTOR",COMMAND_STRING_BUFSIZE); break;
    case COMMAND_SIM_STOP:              (void)strncpy(pcCommandString,"COMMAND_SIM_STOP",COMMAND_STRING_BUFSIZE); break;
    default:                            (void)snprintf(pcCommandString,COMMAND_STRING_BUFSIZE,"COMMAND_\?\?\?(0x%08x)",(unsigned int)eCommand); break;
    }

    pcCommandString[COMMAND_STRING_BUFSIZE - 1] = 0;
    return (const char*)pcCommandString;
}
