/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef R5_PEF_H__
#define R5_PEF_H__

struct LoopState;
typedef struct LoopState LoopState;

struct ProgEnvFuncs;
typedef struct ProgEnvFuncs ProgEnvFuncs;

struct VerifyFlashArgs;
typedef struct VerifyFlashArgs VerifyFlashArgs;

struct PEFContext;
typedef struct PEFContext PEFContext;

struct VerifyFlashArgs {
    uint32_t qConnectionMode;
    uint32_t qBusWidth;
    uint32_t qQspiPeripheralInputSpeedHz;
    uint32_t qPrescaler;
    uint32_t qCacheCoherentDMA;
    uint32_t qPEBSize;
    uint32_t qNumPEBs;
    uint32_t qSubsectorSize;
    uint32_t qNumSubsectors;    /* Inferred */
};

struct LoopState {
    bool yFlashVerified;
    uint32_t qCS;
    uint32_t qMismatchCount;

    VerifyFlashArgs tVerifyFlashArgsIn;

    /* These are the verified values */
    VerifyFlashArgs tVerifiedFlashArgs;

    uint32_t pqVerifyFlashArgs[8];
};

typedef enum CACHE_REGION {
    CACHE_REGION_COMM_AREA,
    CACHE_REGION_RESP_AREA,
    CACHE_REGION_AUX_AREA,
    CACHE_REGION_PRIV_AREA
} CACHE_REGION;

/*
 * Provided by environment (sim or r5)
 * Return value for all functions is ERR cast to a uint32_t
 */
struct ProgEnvFuncs {
    LoopState tLS;

    PEFContext* pContext;
    Sys* pSys;
    MessageArea* pMA;
    AuxArea* pAA;
    AuxArea* pPA;

    void (*rDataBarrier)(
        ProgEnvFuncs* pPEF,
        CACHE_REGION eCacheRegion
        );
    uint32_t (*rVerifyFlash)(
        ProgEnvFuncs* pPEF,
        OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
        );
    uint32_t (*rErasePEBs)(
        ProgEnvFuncs* pPEF,
        uint32_t qPEBStart,
        uint32_t qPEBEnd,
        OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
        );
    uint32_t (*rReadPEB)(
        ProgEnvFuncs* pPEF,
        uint32_t qPEBNumber,
        OUTPUT_PARAMETER volatile uint32_t* pqErrBSP,
        OUTPUT_PARAMETER uint8_t* pbDst
        );
    uint32_t (*rWritePEB)(
        ProgEnvFuncs* pPEF,
        uint32_t qPEBNumber,
        const uint8_t* pbSrc,
        OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
        );
    uint32_t (*rGetMultibootBytes)(
        ProgEnvFuncs* pPEF,
        OUTPUT_PARAMETER volatile uint32_t* pqErrBSP,
        OUTPUT_PARAMETER uint32_t* pqMultibootBytes
        );
};

extern void r5InitPEF(ProgEnvFuncs* pPEF);
extern bool r5StepPEF(ProgEnvFuncs* pPEF);

#endif /* R5_PEF_H__ */
