/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "r5_app.h"

/* ------------------------------------------------------------------ */


/* ------------------------------------------------------------------ */
#ifdef DO_TRACE
void InitUart(void)
{
#ifndef DO_TRACE_CORESIGHT
    XUartPsv tUartPsv;
    XUartPsv_Config* Config;
    int Status;
    
    Config = XUartPsv_LookupConfig(0);
    if( NULL == Config) {
        return;
    }
    Status = XUartPsv_CfgInitialize(&tUartPsv, Config, Config->BaseAddress);
    if (Status != XST_SUCCESS) {
        return;
    }
#endif
}
#endif

/* ------------------------------------------------------------------ */

bool IsVerboseEnabled(Sys* pSys)
{
    return false;
}

bool IsSyslogEnabled(Sys* pSys)
{
    return 0;
}

/* ------------------------------------------------------------------ */

void InitSys(Sys* pSys)
{
    TRACE();
    r5InitPEFH(&pSys->tPEFContext);
    TRACE();
    r5InitMultibootState(&pSys->tMultibootState);
    TRACE();
}

int main(void)
{
    Sys tSys;
    ProgEnvFuncs* pPEF;
    MultibootState* pMS;

    WAIT_FOR_PLM_DONE();

#ifdef DO_TRACE
    InitUart();
    TRACE_PRINT("Build info: %s %s\n\r",__DATE__,__TIME__);
#endif
    TRACE();
    InitSys(&tSys);
    TRACE();

    pPEF = &tSys.tPEFContext.tPEF;
    pMS = &tSys.tMultibootState;
    for(;;) {
        r5SenseMultibootInputs(pMS);

        if(pMS->yMultibootRequest || (PCIE_STATE_DOWN == pMS->ePcieState)) {
            TRACE();
            r5PerformMultiboot(pMS);
            __builtin_unreachable();
            for(;;) {
                ;
            }
        } else if(PCIE_STATE_UP == pMS->ePcieState) {
            TRACE();
            r5StepPEF(pPEF);
        }
    }
    return 0;
}
