/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdint.h>
#include <stdbool.h>

#define FILESIZE    (65536*17+4096*3+23)
int main(int argc,const char* argv[])
{
    int i,j;
    ssize_t w;
    int iCsvFile;
    int iDataFile;
    int iFlags;
    int iMode;
    char pcFileName[32];
    uint8_t pbBuf[16];
    uint8_t b;
    uint8_t pbId[128+1];

    if(argc != 3) {
        fprintf(stderr,"Usage:\n\t%s CSVFILE IMAGEDIR\n\tCSVFILE will be overwritten\n\tThe specified DIR must exist\n",argv[0]);
        return -1;
    }
    iFlags = 0
        | O_WRONLY
        | O_TRUNC
        | O_CREAT
        ;
    iMode = 0
        | S_IRUSR
        | S_IWUSR
        | S_IRGRP
        | S_IROTH
        ;

    iCsvFile = open(argv[1],iFlags,iMode);
    if(iCsvFile < 0) {
        perror("open");
        return -1;
    }

    for(i = 0; i < 16; i++) {
        sprintf(pcFileName,"%s/p%02x.dat",argv[2],i);

        iDataFile = open(pcFileName,iFlags,iMode);
        if(iDataFile < 0) {
            perror("open");
            return -1;
        }

        for(b = i, j = FILESIZE; j > 0; ) {
            memset(pbBuf,((uint8_t)b << 8) | (uint8_t)b,16);
            b++;
            if(j > 16) {
                w = 16;
            } else {
                w = j;
            }
            if(w != write(iDataFile,pbBuf,w)) {
                perror("write");
                return -1;
            }
            j -= w;
        }

        (void)close(iDataFile);

        memset(pbId,'A'+i,128);
        pbId[128] = 0;
        dprintf(iCsvFile,"%s,prom,%s\n",strchr(pcFileName,'/') + 1,pbId);
    }
    (void)close(iCsvFile);
    return 0;
}
